﻿using AutoWorkshopContracts.BindingModels;
using AutoWorkshopContracts.BusinessLogicContracts;
using AutoWorkshopContracts.SearchModels;
using AutoWorkshopContracts.StoragesContracts;
using AutoWorkshopContracts.ViewModels;
using Microsoft.Extensions.Logging;

namespace AutoWorkshopBusinessLogic.BusinessLogics
{
    public class RepairLogic : IRepairLogic
    {
        private readonly ILogger _logger;

        private readonly IRepairStorage _repairStorage;

        public RepairLogic(ILogger<RepairLogic> Logger, IRepairStorage RepairStorage)
        {
            _logger = Logger;
            _repairStorage = RepairStorage;
        }

        public List<RepairViewModel>? ReadList(RepairSearchModel? Model)
        {
            _logger.LogInformation("ReadList. RepairName:{RepairName}. Id:{Id}",
                Model?.RepairName, Model?.Id);

            var List = Model is null ? _repairStorage.GetFullList() : _repairStorage.GetFilteredList(Model);

            if (List is null)
            {
                _logger.LogWarning("ReadList return null list");
                return null;
            }

            _logger.LogInformation("ReadList. Count:{Count}", List.Count);
            return List;
        }

        public RepairViewModel? ReadElement(RepairSearchModel? Model)
        {
            if (Model is null)
                throw new ArgumentNullException(nameof(Model));

            _logger.LogInformation("ReadElement. RepairName:{RepairName}. Id:{Id}",
                Model.RepairName, Model.Id);

            var Element = _repairStorage.GetElement(Model);

            if (Element is null)
            {
                _logger.LogWarning("ReadElement element not found");
                return null;
            }

            _logger.LogInformation("ReadElement element found. Id:{Id}", Element.Id);
            return Element;
        }

        public bool Create(RepairBindingModel Model)
        {
            CheckModel(Model);

            if (_repairStorage.Insert(Model) is null)
            {
                _logger.LogWarning("Insert operation failed");
                return false;
            }

            return true;
        }

        public bool Update(RepairBindingModel Model)
        {
            CheckModel(Model);

            if (_repairStorage.Update(Model) is null)
            {
                _logger.LogWarning("Update operation failed");
                return false;
            }

            return true;
        }

        public bool Delete(RepairBindingModel Model)
        {
            CheckModel(Model);
            _logger.LogInformation("Delete. Id:{Id}", Model.Id);

            if (_repairStorage.Delete(Model) is null)
            {
                _logger.LogWarning("Delete operation failed");
                return false;
            }

            return true;
        }

        private void CheckModel(RepairBindingModel Model, bool WithParams = true)
        {
            if (Model == null)
                throw new ArgumentNullException(nameof(Model));

            if (!WithParams)
                return;

            if (string.IsNullOrEmpty(Model.RepairName))
                throw new ArgumentNullException("Нет названия ремонта", nameof(Model.RepairName));

            if (Model.Price <= 0)
                throw new ArgumentNullException("Стоимость ремонта должна быть больше 0", nameof(Model.Price));

            _logger.LogInformation("Repair. RepairName:{RepairName}. Price:{Price}. Id: {Id}",
                Model.RepairName, Model.Price, Model.Id);

            var Element = _repairStorage.GetElement(new RepairSearchModel
                {
                    RepairName = Model.RepairName
                });

            if (Element != null && Element.Id != Model.Id)
                throw new InvalidOperationException("Ремонт с таким названием уже есть");
        }
    }
}
