﻿using AutoWorkshopContracts.BindingModels;
using AutoWorkshopContracts.ViewModels;
using AutoWorkshopDataModels.Models;
using System.ComponentModel.DataAnnotations;
using System.ComponentModel.DataAnnotations.Schema;

namespace AutoWorkshopDatabaseImplement.Models
{
    public class Shop : IShopModel
    {
        public int Id { get; set; }

        [Required]
        public string ShopName { get; set; } = string.Empty;

        [Required]
        public string Address { get; set; } = string.Empty;

        [Required]
        public DateTime OpeningDate { get; set; }

        [Required]
        public int RepairsMaxCount { get; set; }

        [ForeignKey("ShopId")]
        public List<ShopRepair> Repairs { get; set; } = new();

        private Dictionary<int, (IRepairModel, int)>? _shopRepairs = null;

        [NotMapped]
        public Dictionary<int, (IRepairModel, int)> ShopRepairs
        {
            get
            {
                if (_shopRepairs == null)
                {
                    if (_shopRepairs == null)
                    {
                        _shopRepairs = Repairs.ToDictionary(ShopRep => ShopRep.RepairId, ShopRep => (ShopRep.Repair as IRepairModel, ShopRep.Count));
                    }                    
                    return _shopRepairs;
                }
                
                return _shopRepairs;
            }
        }

        public static Shop Create(AutoWorkshopDatabase Context, ShopBindingModel Model)
        {
            return new Shop()
            {
                Id = Model.Id,
                ShopName = Model.ShopName,
                Address = Model.Address,
                OpeningDate = Model.OpeningDate,
                Repairs = Model.ShopRepairs.Select(x => new ShopRepair
                {
                    Repair = Context.Repairs.First(y => y.Id == x.Key),
                    Count = x.Value.Item2
                }).ToList(),
                RepairsMaxCount = Model.RepairsMaxCount
            };
        }

        public void Update(ShopBindingModel Model)
        {
            ShopName = Model.ShopName;
            Address = Model.Address;
            OpeningDate = Model.OpeningDate;
            RepairsMaxCount = Model.RepairsMaxCount;
        }

        public ShopViewModel GetViewModel => new()
        {
            Id = Id,
            ShopName = ShopName,
            Address = Address,
            OpeningDate = OpeningDate,
            ShopRepairs = ShopRepairs,
            RepairsMaxCount = RepairsMaxCount
        };

        public void UpdateRepairs(AutoWorkshopDatabase Context, ShopBindingModel Model)
        {
            var ShopRepairs = Context.ShopRepairs
                .Where(rec => rec.ShopId == Model.Id)
                .ToList();
            
            if (ShopRepairs != null && ShopRepairs.Count > 0)
            {
                Context.ShopRepairs.RemoveRange(ShopRepairs.Where(rec => !Model.ShopRepairs.ContainsKey(rec.RepairId)));
                Context.SaveChanges();

				ShopRepairs = Context.ShopRepairs.Where(rec => rec.ShopId == Model.Id).ToList();

				foreach (var RepairToUpdate in ShopRepairs)
                {
                    RepairToUpdate.Count = Model.ShopRepairs[RepairToUpdate.RepairId].Item2;
                    Model.ShopRepairs.Remove(RepairToUpdate.RepairId);
                }
                
                Context.SaveChanges();
            }
            
            var Shop = Context.Shops.First(x => x.Id == Id);
            
            foreach (var ShopRepair in Model.ShopRepairs)
            {
                Context.ShopRepairs.Add(new ShopRepair
                {
                    Shop = Shop,
                    Repair = Context.Repairs.First(x => x.Id == ShopRepair.Key),
                    Count = ShopRepair.Value.Item2
                });
                
                Context.SaveChanges();
            }
            
            _shopRepairs = null;
        }

        public void RepairsDictionatyUpdate(AutoWorkshopDatabase Context)
        {
            UpdateRepairs(Context, new ShopBindingModel
            {
                Id = Id,
                ShopRepairs = ShopRepairs
            });
        }
    }
}
