﻿using AutoWorkshopContracts.BindingModels;
using AutoWorkshopContracts.SearchModels;
using AutoWorkshopContracts.StoragesContracts;
using AutoWorkshopContracts.ViewModels;
using AutoWorkshopFileImplement.Models;

namespace AutoWorkshopFileImplement.Implements
{
    public class ShopStorage : IShopStorage
    {
        private readonly DataFileSingleton _source;

        public ShopStorage()
        {
            _source = DataFileSingleton.GetInstance();
        }

        public List<ShopViewModel> GetFullList()
        {
            return _source.Shops.Select(x => x.GetViewModel).ToList();
        }

        public List<ShopViewModel> GetFilteredList(ShopSearchModel Model)
        {
            if (string.IsNullOrEmpty(Model.ShopName))
                return new();

            return _source.Shops
                .Where(x => x.ShopName
                .Contains(Model.ShopName))
                .Select(x => x.GetViewModel)
                .ToList();
        }

        public ShopViewModel? GetElement(ShopSearchModel Model)
        {
            if (string.IsNullOrEmpty(Model.ShopName) && !Model.Id.HasValue)
                return null;

            return _source.Shops.FirstOrDefault(x =>
                (!string.IsNullOrEmpty(Model.ShopName) && x.ShopName == Model.ShopName) ||
                (Model.Id.HasValue && x.Id == Model.Id))?
                .GetViewModel;
        }

        public ShopViewModel? Insert(ShopBindingModel Model)
        {
            Model.Id = _source.Shops.Count > 0 ? _source.Shops.Max(x => x.Id) + 1 : 1;
            
            var NewShop = Shop.Create(Model);
            if (NewShop == null)
                return null;
            
            _source.Shops.Add(NewShop);
            _source.SaveShops();
            
            return NewShop.GetViewModel;
        }

        public ShopViewModel? Update(ShopBindingModel Model)
        {
            var Shop = _source.Shops.FirstOrDefault(x => x.Id == Model.Id);
            if (Shop == null)
                return null;

            Shop.Update(Model);
            _source.SaveShops();
            
            return Shop.GetViewModel;
        }

        public ShopViewModel? Delete(ShopBindingModel Model)
        {
            var Shop = _source.Shops.FirstOrDefault(x => x.Id == Model.Id);

            if (Shop == null)
                return null;

            _source.Shops.Remove(Shop);
            _source.SaveShops();
            
            return Shop.GetViewModel;
        }

        public bool Sell(SupplySearchModel Model)
        {
            if (Model == null || !Model.RepairId.HasValue || !Model.Count.HasValue)
                return false;
            
            int TotalRepairsNum = _source.Shops
                .Select(x => x.Repairs.ContainsKey(Model.RepairId.Value) ? x.Repairs[Model.RepairId.Value] : 0)
                .Sum();
            if (TotalRepairsNum < Model.Count)
                return false;

            var ShopsWithDesiredRepair = _source.Shops
                .Where(x => x.Repairs.ContainsKey(Model.RepairId.Value))
                .OrderByDescending(x => x.Repairs[Model.RepairId.Value])
                .ToList();
            
            foreach (var Shop in ShopsWithDesiredRepair)
            {
                int Slack = Model.Count.Value - Shop.Repairs[Model.RepairId.Value];
                
                if (Slack > 0)
                {
                    Shop.Repairs.Remove(Model.RepairId.Value);
                    Shop.RepairsUpdate();
                    
                    Model.Count = Slack;
                }
                else
                {
                    if (Slack == 0)
                        Shop.Repairs.Remove(Model.RepairId.Value);
                    else
                        Shop.Repairs[Model.RepairId.Value] = -Slack;

                    Shop.RepairsUpdate();
                    _source.SaveShops();
                    
                    return true;
                }
            }
            
            _source.SaveShops();
            return false;
        }

        public bool RestockingShops(SupplyBindingModel Model)
        {
            int TotalFreeSpaceNum = _source.Shops
                .Select(x => x.RepairsMaxCount - x.ShopRepairs
                    .Select(y => y.Value.Item2)
                    .Sum())
                .Sum();

            if (TotalFreeSpaceNum < Model.Count)
                return false;

            foreach (Shop Shop in _source.Shops)
            {
                int FreeSpaceNum = Shop.RepairsMaxCount - Shop.ShopRepairs.Select(x => x.Value.Item2).Sum();
                
                if (FreeSpaceNum <= 0)
                    continue;

                FreeSpaceNum = Math.Min(FreeSpaceNum, Model.Count);
                Model.Count -= FreeSpaceNum;
                
                if (Shop.Repairs.ContainsKey(Model.RepairId))
                    Shop.Repairs[Model.RepairId] += FreeSpaceNum;
                else
                    Shop.Repairs.Add(Model.RepairId, FreeSpaceNum);

                Shop.RepairsUpdate();
                
                if (Model.Count == 0)
                {
                    _source.SaveShops();
                    return true;
                }
            }
            
            return false;
        }
    }
}
