﻿using AutoWorkshopContracts.BindingModels;
using AutoWorkshopContracts.BusinessLogicContracts;
using AutoWorkshopContracts.SearchModels;
using AutoWorkshopDataModels.Models;
using Microsoft.Extensions.Logging;

namespace AutoWorkshopView.Forms
{
    public partial class FormRepair : Form
    {
        private readonly ILogger _logger;

        private readonly IRepairLogic _logic;

        private int? _id;

        private Dictionary<int, (IComponentModel, int)> _repairComponents;

        public int Id { set { _id = value; } }

        public FormRepair(ILogger<FormRepair> Logger, IRepairLogic Logic)
        {
            InitializeComponent();
            _logger = Logger;
            _logic = Logic;
            _repairComponents = new Dictionary<int, (IComponentModel, int)>();
        }

        private void FormRepair_Load(object sender, EventArgs e)
        {
            if (_id.HasValue)
            {
                _logger.LogInformation("Загрузка ремонта");

                try
                {
                    var View = _logic.ReadElement(new RepairSearchModel
                    {
                        Id = _id.Value
                    });
                    
                    if (View != null)
                    {
                        NameTextBox.Text = View.RepairName;
                        PriceTextBox.Text = View.Price.ToString();
                        
                        _repairComponents = View.RepairComponents ?? new Dictionary<int, (IComponentModel, int)>();
                        LoadData();
                    }
                }
                catch (Exception ex)
                {
                    _logger.LogError(ex, "Ошибка загрузки ремонта");
                    MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
        }

        private void LoadData()
        {
            _logger.LogInformation("Загрузка компонентов ремонта");
            
            try
            {
                if (_repairComponents != null)
                {
                    DataGridView.Rows.Clear();
                    foreach (var Comp in _repairComponents)
                    {
                        DataGridView.Rows.Add(new object[] { Comp.Key, Comp.Value.Item1.ComponentName, Comp.Value.Item2 });
                    }
                    
                    PriceTextBox.Text = CalcPrice().ToString();
                }
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ошибка загрузки компонентов ремонта");
                MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void AddButton_Click(object sender, EventArgs e)
        {
            var Service = Program.ServiceProvider?.GetService(typeof(FormRepairComponent));
            
            if (Service is FormRepairComponent Form)
            {
                if (Form.ShowDialog() == DialogResult.OK)
                {
                    if (Form.ComponentModel == null)
                    {
                        return;
                    }

                    _logger.LogInformation("Добавление нового компонента: {ComponentName} - {Count}", Form.ComponentModel.ComponentName, Form.Count);
                    
                    if (_repairComponents.ContainsKey(Form.Id))
                    {
                        _repairComponents[Form.Id] = (Form.ComponentModel, Form.Count);
                    }
                    else
                    {
                        _repairComponents.Add(Form.Id, (Form.ComponentModel, Form.Count));
                    }
                    
                    LoadData();
                }
            }
        }

        private void UpdateButton_Click(object sender, EventArgs e)
        {
            if (DataGridView.SelectedRows.Count == 1)
            {
                var service = Program.ServiceProvider?.GetService(typeof(FormRepairComponent));
                if (service is FormRepairComponent Form)
                {
                    int id = Convert.ToInt32(DataGridView.SelectedRows[0].Cells[0].Value);
                    Form.Id = id;                    
                    Form.Count = _repairComponents[id].Item2;
                    
                    if (Form.ShowDialog() == DialogResult.OK)
                    {
                        if (Form.ComponentModel == null)
                        {
                            return;
                        }
                        
                        _logger.LogInformation("Изменение компонента: {ComponentName} - {Count}", Form.ComponentModel.ComponentName, Form.Count);
                        _repairComponents[Form.Id] = (Form.ComponentModel, Form.Count);
                       
                        LoadData();
                    }
                }
            }
        }

        private void DeleteButton_Click(object sender, EventArgs e)
        {
            if (DataGridView.SelectedRows.Count == 1)
            {
                if (MessageBox.Show("Удалить запись?", "Вопрос", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.Yes)
                {
                    try
                    {
                        _logger.LogInformation("Удаление компонента: {ComponentName} - {Count}", DataGridView.SelectedRows[0].Cells[1].Value,
                            DataGridView.SelectedRows[0].Cells[2].Value);
                        
                        _repairComponents?.Remove(Convert.ToInt32(DataGridView.SelectedRows[0].Cells[0].Value));
                    }
                    catch (Exception ex)
                    {
                        MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    }
                    
                    LoadData();
                }
            }
        }

        private void RefreshButton_Click(object sender, EventArgs e)
        {
            LoadData();
        }

        private void SaveButton_Click(object sender, EventArgs e)
        {

            if (string.IsNullOrEmpty(NameTextBox.Text))
            {
                MessageBox.Show("Заполните название", "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            
            if (string.IsNullOrEmpty(PriceTextBox.Text))
            {
                MessageBox.Show("Заполните цену", "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            
            if (_repairComponents == null || _repairComponents.Count == 0)
            {
                MessageBox.Show("Заполните компоненты", "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            
            _logger.LogInformation("Сохранение ремонта");
            
            try
            {
                var Мodel = new RepairBindingModel
                {
                    Id = _id ?? 0,
                    RepairName = NameTextBox.Text,
                    Price = Convert.ToDouble(PriceTextBox.Text),
                    RepairComponents = _repairComponents
                };

                var OperationResult = _id.HasValue ? _logic.Update(Мodel) : _logic.Create(Мodel);
                
                if (!OperationResult)
                {
                    throw new Exception("Ошибка при сохранении. Дополнительная информация в логах.");
                }
                
                MessageBox.Show("Сохранение прошло успешно", "Сообщение", MessageBoxButtons.OK, MessageBoxIcon.Information);
                DialogResult = DialogResult.OK;
                Close();
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ошибка сохранения ремонта");
                MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void ButtonCancel_Click(object sender, EventArgs e)
        {
            DialogResult = DialogResult.Cancel;
            Close();
        }

        private double CalcPrice()
        {
            double Price = 0;
            
            foreach (var Elem in _repairComponents)
            {
                Price += ((Elem.Value.Item1?.Cost ?? 0) * Elem.Value.Item2);
            }
            
            return Math.Round(Price * 1.1, 2);
        }
    }
}
