﻿using AutoWorkshopContracts.BindingModels;
using AutoWorkshopContracts.BusinessLogicsContracts;
using AutoWorkshopContracts.SearchModels;
using AutoWorkshopDataModels.Models;
using Microsoft.Extensions.Logging;
using System.Windows.Forms;

namespace AutoWorkshopView.Forms.Shop
{
    public partial class FormShop : Form
    {
        private readonly ILogger _logger;
        private readonly IShopLogic _logic;
        
        private int? _id;
        
        public int Id { set { _id = value; } }
        
        private Dictionary<int, (IRepairModel, int)> _shopRepairs;
        private DateTime? _openingDate = null;

        public FormShop(ILogger<FormShop> Logger, IShopLogic Logic)
        {
            InitializeComponent();
            
            _logger = Logger;
            _logic = Logic;
            _shopRepairs = new Dictionary<int, (IRepairModel, int)>();
        }

        private void FormShop_Load(object sender, EventArgs e)
        {
            if (_id.HasValue)
            {
                _logger.LogInformation("Загрузка магазина");
                
                try
                {
                    var View = _logic.ReadElement(new ShopSearchModel
                    {
                        Id = _id.Value
                    });
                    
                    if (View != null)
                    {
                        NameTextBox.Text = View.ShopName;
                        AddressTextBox.Text = View.Address;
                        OpenDateTimePicker.Value = View.OpeningDate;
                        RepairsMaxCountNumericUp.Value = View.RepairsMaxCount;
                        _shopRepairs = View.ShopRepairs ?? new Dictionary<int, (IRepairModel, int)>();
                        
                        LoadData();
                    }
                }
                catch (Exception ex)
                {
                    _logger.LogError(ex, "Ошибка загрузки магазина");
                    MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
        }

        private void LoadData()
        {
            _logger.LogInformation("Загрузка ремонтов в магазине");
            
            try
            {
                if (_shopRepairs != null)
                {
                    ViewDataGrid.Rows.Clear();
                    foreach (var ShopRepair in _shopRepairs)
                    {
                        ViewDataGrid.Rows.Add(new object[] { ShopRepair.Key, ShopRepair.Value.Item1.RepairName, ShopRepair.Value.Item2 });
                    }
                }
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ошибка загрузки изделий магазина");
                MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void SaveButton_Click(object sender, EventArgs e)
        {
            if (string.IsNullOrEmpty(NameTextBox.Text))
            {
                MessageBox.Show("Заполните название", "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            
            if (string.IsNullOrEmpty(AddressTextBox.Text))
            {
                MessageBox.Show("Заполните адрес", "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            
            _logger.LogInformation("Сохранение магазина");
            
            try
            {
                var Model = new ShopBindingModel
                {
                    Id = _id ?? 0,
                    ShopName = NameTextBox.Text,
                    Address = AddressTextBox.Text,
                    OpeningDate = OpenDateTimePicker.Value,
                    RepairsMaxCount = (int)RepairsMaxCountNumericUp.Value,
                };
                
                var OperationResult = _id.HasValue ? _logic.Update(Model) : _logic.Create(Model);
                if (!OperationResult)
                {
                    throw new Exception("Ошибка при сохранении. Дополнительная информация в логах.");
                }
                
                MessageBox.Show("Сохранение прошло успешно", "Сообщение", MessageBoxButtons.OK, MessageBoxIcon.Information);
                DialogResult = DialogResult.OK;
                
                Close();
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ошибка сохранения магазина");
                MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void CancelButton_Click(object sender, EventArgs e)
        {
            DialogResult = DialogResult.Cancel;
            Close();
        }
    }
}
